/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef JNPP_COMMON
#define JNPP_COMMON

#define JNPP_STATUS_INTERNAL_ERROR       0x10000001

#include <stdlib.h>
#include <jni.h>
#include <nppi.h>
#include <npps.h>

#include "Logger.hpp"
#include "JNIUtils.hpp"
#include "PointerUtils.hpp"


extern jfieldID cuComplex_x; // float
extern jfieldID cuComplex_y; // float

extern jfieldID cuDoubleComplex_x; // double
extern jfieldID cuDoubleComplex_y; // double

extern jfieldID NppiPoint_x; // int
extern jfieldID NppiPoint_y; // int

extern jfieldID NppiSize_width; // int
extern jfieldID NppiSize_height; // int

extern jfieldID NppiRect_x; // int
extern jfieldID NppiRect_y; // int
extern jfieldID NppiRect_width; // int
extern jfieldID NppiRect_height; // int


extern jfieldID Npp16sc_re; // short
extern jfieldID Npp16sc_im; // short

extern jfieldID Npp32sc_re; // int
extern jfieldID Npp32sc_im; // int

extern jfieldID Npp64sc_re; // long
extern jfieldID Npp64sc_im; // long

extern jfieldID Npp32fc_re; // float
extern jfieldID Npp32fc_im; // float

extern jfieldID Npp64fc_re; // double
extern jfieldID Npp64fc_im; // double


// Class and method ID for TypedPointer and its constructor
extern jclass TypedPointer_Class;
extern jmethodID TypedPointer_Constructor;


jobject createPointer(JNIEnv *env, jlong pointerValue);
bool initNative(JNIEnv *env, jobject input, NppiPoint &output);
bool initNative(JNIEnv *env, jobject input, NppiSize &output);
bool initNative(JNIEnv *env, jobject input, NppiRect &output);

bool initNative(JNIEnv *env, jobjectArray input, NppiDecodeHuffmanSpec* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, NppiEncodeHuffmanSpec* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, NppiSize output[], int size);

#include "JNpp_common_complex.hpp"
#include "JNpp_common_arrays2D.hpp"
#include "JNpp_common_arrays1D.hpp"
#include "JNpp_common_objectArrays.hpp"






#endif