/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"

// TODO: Many of these methods have been auto-generated and could 
// be replaced by more generic ones 

//=== Complex number arrays ==================================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16sc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16sc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32sc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32sc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32fc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32fc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp64fc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64fc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}





//=== Primitive pointer arrays ===============================================


/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp8s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp8s*)getPointer(env, inputElement);
    }
    return true;
}


/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp8s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp8s*)getPointer(env, inputElement);
    }
    return true;
}


/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp8u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp8u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp8u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp8u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp16s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp16s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp16s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp16u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp16u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp16u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp32s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp32s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp32s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp32f*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp32f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp32f*)getPointer(env, inputElement);
    }
    return true;
}

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp64f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp64f*)getPointer(env, inputElement);
    }
    return true;
}


/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp64f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp64f*)getPointer(env, inputElement);
    }
    return true;
}

