/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package jcuda.jnpp;
/**
 * Error Status Codes.
 *
 * Almost all NPP function return error-status information using
 * these return codes.
 * Negative return codes indicate errors, positive return codes indicate
 * warnings, a return code of 0 indicates success.
 */
class NppStatus
{

    // negative return-codes indicate errors

    public static final int NPP_NOT_SUPPORTED_MODE_ERROR = -9999;
    public static final int NPP_INVALID_HOST_POINTER_ERROR = -1032;
    public static final int NPP_INVALID_DEVICE_POINTER_ERROR = -1031;
    public static final int NPP_LUT_PALETTE_BITSIZE_ERROR = -1030;
    public static final int NPP_ZC_MODE_NOT_SUPPORTED_ERROR = -1028;
    public static final int NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY = -1027;
    public static final int NPP_TEXTURE_BIND_ERROR = -1024;
    public static final int NPP_WRONG_INTERSECTION_ROI_ERROR = -1020;
    public static final int NPP_HAAR_CLASSIFIER_PIXEL_MATCH_ERROR = -1006;
    public static final int NPP_MEMFREE_ERROR = -1005;
    public static final int NPP_MEMSET_ERROR = -1004;
    public static final int NPP_MEMCPY_ERROR = -1003;
    public static final int NPP_ALIGNMENT_ERROR = -1002;
    public static final int NPP_CUDA_KERNEL_EXECUTION_ERROR = -1000;
    public static final int NPP_ROUND_MODE_NOT_SUPPORTED_ERROR = -213;
    public static final int NPP_QUALITY_INDEX_ERROR = -210;
    public static final int NPP_RESIZE_NO_OPERATION_ERROR = -201;
    public static final int NPP_OVERFLOW_ERROR = -109;
    public static final int NPP_NOT_EVEN_STEP_ERROR = -108;
    /** Number of levels for histogram is less than 2 */
    public static final int NPP_HISTOGRAM_NUMBER_OF_LEVELS_ERROR = -107;
    public static final int NPP_LUT_NUMBER_OF_LEVELS_ERROR = -106;
    public static final int NPP_CHANNEL_ORDER_ERROR = -60;
    public static final int NPP_ZERO_MASK_VALUE_ERROR = -59;
    public static final int NPP_QUADRANGLE_ERROR = -58;
    public static final int NPP_RECTANGLE_ERROR = -57;
    public static final int NPP_COEFFICIENT_ERROR = -56;
    public static final int NPP_NUMBER_OF_CHANNELS_ERROR = -53;
    /** Channel of interest is not 1, 2, or 3 */
    public static final int NPP_COI_ERROR = -52;
    public static final int NPP_DIVISOR_ERROR = -51;
    public static final int NPP_CHANNEL_ERROR = -47;
    /** Stride is less than the row length */
    public static final int NPP_STRIDE_ERROR = -37;
    /** Anchor point is outside mask */
    public static final int NPP_ANCHOR_ERROR = -34;
    public static final int NPP_MASK_SIZE_ERROR = -33;
    public static final int NPP_RESIZE_FACTOR_ERROR = -23;
    public static final int NPP_INTERPOLATION_ERROR = -22;
    public static final int NPP_MIRROR_FLIP_ERROR = -21;
    public static final int NPP_MOMENT_00_ZERO_ERROR = -20;
    public static final int NPP_THRESHOLD_NEGATIVE_LEVEL_ERROR = -19;
    public static final int NPP_THRESHOLD_ERROR = -18;
    public static final int NPP_CONTEXT_MATCH_ERROR = -17;
    public static final int NPP_FFT_FLAG_ERROR = -16;
    public static final int NPP_FFT_ORDER_ERROR = -15;
    /**  Step is less or equal zero */
    public static final int NPP_STEP_ERROR = -14;
    public static final int NPP_SCALE_RANGE_ERROR = -13;
    public static final int NPP_DATA_TYPE_ERROR = -12;
    public static final int NPP_OUT_OFF_RANGE_ERROR = -11;
    public static final int NPP_DIVIDE_BY_ZERO_ERROR = -10;
    public static final int NPP_MEMORY_ALLOCATION_ERR = -9;
    public static final int NPP_NULL_POINTER_ERROR = -8;
    public static final int NPP_RANGE_ERROR = -7;
    public static final int NPP_SIZE_ERROR = -6;
    public static final int NPP_BAD_ARGUMENT_ERROR = -5;
    public static final int NPP_NO_MEMORY_ERROR = -4;
    public static final int NPP_NOT_IMPLEMENTED_ERROR = -3;
    public static final int NPP_ERROR = -2;
    public static final int NPP_ERROR_RESERVED = -1;

    // success

    /** Error free operation */
    public static final int NPP_NO_ERROR = 0;

    /** Successful operation (same as NPP_NO_ERROR) */
    public static final int NPP_SUCCESS = NPP_NO_ERROR;


    // positive return-codes indicate warnings
    public static final int NPP_NO_OPERATION_WARNING = 1;
    public static final int NPP_DIVIDE_BY_ZERO_WARNING = 6;
    
    /** 
     * Indicates that the quadrangle passed to one of affine 
     * warping functions doesn't have necessary properties. First 
     * 3 vertices are used, the fourth vertex discarded. 
     */
    public static final int NPP_AFFINE_QUAD_INCORRECT_WARNING = 28;
    public static final int NPP_WRONG_INTERSECTION_ROI_WARNING = 29;
    
    /**  
     * The given quadrangle has no intersection with either the 
     * source or destination ROI. Thus no operation was performed. 
     */
    public static final int NPP_WRONG_INTERSECTION_QUAD_WARNING = 30;
    public static final int NPP_DOUBLE_SIZE_WARNING = 35;

    /**
     * Speed reduction due to uncoalesced memory accesses warning.
     */
    public static final int NPP_MISALIGNED_DST_ROI_WARNING = 10000;


    /**
     * Alias for NPP_AFFINE_QUAD_INCORRECT_WARNING resembling the IPP warning more closely.
     */
    public static final int NPP_AFFINE_QUAD_CHANGED_WARNING = NPP_AFFINE_QUAD_INCORRECT_WARNING;


    /**
     * Indicates that in case of 422/411/420 sampling the ROI width/height was modified for proper processing.
     */
    public static final int NPP_ADJUSTED_ROI_SIZE_WARNING = 5;

    /**
     * Returns the String identifying the given NppStatus
     *
     * @param m The NppStatus
     * @return The String identifying the given NppStatus
     */
    public static String stringFor(int m)
    {
        switch (m)
        {
            case NPP_NOT_SUPPORTED_MODE_ERROR: return "NPP_NOT_SUPPORTED_MODE_ERROR";
            case NPP_INVALID_HOST_POINTER_ERROR: return "NPP_INVALID_HOST_POINTER_ERROR";
            case NPP_INVALID_DEVICE_POINTER_ERROR: return "NPP_INVALID_DEVICE_POINTER_ERROR";
            case NPP_LUT_PALETTE_BITSIZE_ERROR: return "NPP_LUT_PALETTE_BITSIZE_ERROR";
            case NPP_ZC_MODE_NOT_SUPPORTED_ERROR: return "NPP_ZC_MODE_NOT_SUPPORTED_ERROR";
            case NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY: return "NPP_NOT_SUFFICIENT_COMPUTE_CAPABILITY";
            case NPP_TEXTURE_BIND_ERROR: return "NPP_TEXTURE_BIND_ERROR";
            case NPP_WRONG_INTERSECTION_ROI_ERROR: return "NPP_WRONG_INTERSECTION_ROI_ERROR";
            case NPP_HAAR_CLASSIFIER_PIXEL_MATCH_ERROR: return "NPP_HAAR_CLASSIFIER_PIXEL_MATCH_ERROR";
            case NPP_MEMFREE_ERROR: return "NPP_MEMFREE_ERROR";
            case NPP_MEMSET_ERROR: return "NPP_MEMSET_ERROR";
            case NPP_MEMCPY_ERROR: return "NPP_MEMCPY_ERROR";
            case NPP_ALIGNMENT_ERROR: return "NPP_ALIGNMENT_ERROR";
            case NPP_CUDA_KERNEL_EXECUTION_ERROR: return "NPP_CUDA_KERNEL_EXECUTION_ERROR";
            case NPP_ROUND_MODE_NOT_SUPPORTED_ERROR: return "NPP_ROUND_MODE_NOT_SUPPORTED_ERROR";
            case NPP_QUALITY_INDEX_ERROR: return "NPP_QUALITY_INDEX_ERROR";
            case NPP_RESIZE_NO_OPERATION_ERROR: return "NPP_RESIZE_NO_OPERATION_ERROR";
            case NPP_OVERFLOW_ERROR: return "NPP_OVERFLOW_ERROR";
            case NPP_NOT_EVEN_STEP_ERROR: return "NPP_NOT_EVEN_STEP_ERROR";
            case NPP_HISTOGRAM_NUMBER_OF_LEVELS_ERROR: return "NPP_HISTOGRAM_NUMBER_OF_LEVELS_ERROR";
            case NPP_LUT_NUMBER_OF_LEVELS_ERROR: return "NPP_LUT_NUMBER_OF_LEVELS_ERROR";
            case NPP_CHANNEL_ORDER_ERROR: return "NPP_CHANNEL_ORDER_ERROR";
            case NPP_ZERO_MASK_VALUE_ERROR: return "NPP_ZERO_MASK_VALUE_ERROR";
            case NPP_QUADRANGLE_ERROR: return "NPP_QUADRANGLE_ERROR";
            case NPP_RECTANGLE_ERROR: return "NPP_RECTANGLE_ERROR";
            case NPP_COEFFICIENT_ERROR: return "NPP_COEFFICIENT_ERROR";
            case NPP_NUMBER_OF_CHANNELS_ERROR: return "NPP_NUMBER_OF_CHANNELS_ERROR";
            case NPP_COI_ERROR: return "NPP_COI_ERROR";
            case NPP_DIVISOR_ERROR: return "NPP_DIVISOR_ERROR";
            case NPP_CHANNEL_ERROR: return "NPP_CHANNEL_ERROR";
            case NPP_STRIDE_ERROR: return "NPP_STRIDE_ERROR";
            case NPP_ANCHOR_ERROR: return "NPP_ANCHOR_ERROR";
            case NPP_MASK_SIZE_ERROR: return "NPP_MASK_SIZE_ERROR";
            case NPP_RESIZE_FACTOR_ERROR: return "NPP_RESIZE_FACTOR_ERROR";
            case NPP_INTERPOLATION_ERROR: return "NPP_INTERPOLATION_ERROR";
            case NPP_MIRROR_FLIP_ERROR: return "NPP_MIRROR_FLIP_ERROR";
            case NPP_MOMENT_00_ZERO_ERROR: return "NPP_MOMENT_00_ZERO_ERROR";
            case NPP_THRESHOLD_NEGATIVE_LEVEL_ERROR: return "NPP_THRESHOLD_NEGATIVE_LEVEL_ERROR";
            case NPP_THRESHOLD_ERROR: return "NPP_THRESHOLD_ERROR";
            case NPP_CONTEXT_MATCH_ERROR: return "NPP_CONTEXT_MATCH_ERROR";
            case NPP_FFT_FLAG_ERROR: return "NPP_FFT_FLAG_ERROR";
            case NPP_FFT_ORDER_ERROR: return "NPP_FFT_ORDER_ERROR";
            case NPP_STEP_ERROR: return "NPP_STEP_ERROR";
            case NPP_SCALE_RANGE_ERROR: return "NPP_SCALE_RANGE_ERROR";
            case NPP_DATA_TYPE_ERROR: return "NPP_DATA_TYPE_ERROR";
            case NPP_OUT_OFF_RANGE_ERROR: return "NPP_OUT_OFF_RANGE_ERROR";
            case NPP_DIVIDE_BY_ZERO_ERROR: return "NPP_DIVIDE_BY_ZERO_ERROR";
            case NPP_MEMORY_ALLOCATION_ERR: return "NPP_MEMORY_ALLOCATION_ERR";
            case NPP_NULL_POINTER_ERROR: return "NPP_NULL_POINTER_ERROR";
            case NPP_RANGE_ERROR: return "NPP_RANGE_ERROR";
            case NPP_SIZE_ERROR: return "NPP_SIZE_ERROR";
            case NPP_BAD_ARGUMENT_ERROR: return "NPP_BAD_ARGUMENT_ERROR";
            case NPP_NO_MEMORY_ERROR: return "NPP_NO_MEMORY_ERROR";
            case NPP_NOT_IMPLEMENTED_ERROR: return "NPP_NOT_IMPLEMENTED_ERROR";
            case NPP_ERROR: return "NPP_ERROR";
            case NPP_ERROR_RESERVED: return "NPP_ERROR_RESERVED";
            //case NPP_NO_ERROR: 
            case NPP_SUCCESS: return "NPP_SUCCESS/NPP_NO_ERROR";
            case NPP_NO_OPERATION_WARNING: return "NPP_NO_OPERATION_WARNING";
            case NPP_DIVIDE_BY_ZERO_WARNING: return "NPP_DIVIDE_BY_ZERO_WARNING";
            case NPP_AFFINE_QUAD_INCORRECT_WARNING: return "NPP_AFFINE_QUAD_INCORRECT_WARNING";
            case NPP_WRONG_INTERSECTION_ROI_WARNING: return "NPP_WRONG_INTERSECTION_ROI_WARNING";
            case NPP_WRONG_INTERSECTION_QUAD_WARNING: return "NPP_WRONG_INTERSECTION_QUAD_WARNING";
            case NPP_DOUBLE_SIZE_WARNING: return "NPP_DOUBLE_SIZE_WARNING";
            case NPP_MISALIGNED_DST_ROI_WARNING: return "NPP_MISALIGNED_DST_ROI_WARNING";
        }
        return "INVALID NppStatus: " + m;
    }

    /**
     * Private constructor to prevent instantiation.
     */
    private NppStatus()
    {
    }

}
