/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package jcuda.jnpp;

/**
 * Mask sizes 
 */
public class NppiMaskSize
{
    public static final int NPP_MASK_SIZE_1_X_3 = 0;
    public static final int NPP_MASK_SIZE_1_X_5 = 1;
    public static final int NPP_MASK_SIZE_3_X_1 = 100;
    public static final int NPP_MASK_SIZE_5_X_1 = 101;
    public static final int NPP_MASK_SIZE_3_X_3 = 200;
    public static final int NPP_MASK_SIZE_5_X_5 = 201;
    public static final int NPP_MASK_SIZE_7_X_7 = 400;
    public static final int NPP_MASK_SIZE_9_X_9 = 500;
    public static final int NPP_MASK_SIZE_11_X_11 = 600;
    public static final int NPP_MASK_SIZE_13_X_13 = 700;
    public static final int NPP_MASK_SIZE_15_X_15 = 800;
    
    /**
     * Returns the String identifying the given NppiMaskSize
     *
     * @param m The NppiMaskSize
     * @return The String identifying the given NppiMaskSize
     */
    public static String stringFor(int m)
    {
        switch (m)
        {
            case NPP_MASK_SIZE_1_X_3 : return "NPP_MASK_SIZE_1_X_3";
            case NPP_MASK_SIZE_1_X_5 : return "NPP_MASK_SIZE_1_X_5";
            case NPP_MASK_SIZE_3_X_1 : return "NPP_MASK_SIZE_3_X_1";
            case NPP_MASK_SIZE_5_X_1 : return "NPP_MASK_SIZE_5_X_1";
            case NPP_MASK_SIZE_3_X_3 : return "NPP_MASK_SIZE_3_X_3";
            case NPP_MASK_SIZE_5_X_5 : return "NPP_MASK_SIZE_5_X_5";
            case NPP_MASK_SIZE_7_X_7 : return "NPP_MASK_SIZE_7_X_7";
            case NPP_MASK_SIZE_9_X_9 : return "NPP_MASK_SIZE_9_X_9";
            case NPP_MASK_SIZE_11_X_11 : return "NPP_MASK_SIZE_11_X_11";
            case NPP_MASK_SIZE_13_X_13 : return "NPP_MASK_SIZE_13_X_13";
            case NPP_MASK_SIZE_15_X_15 : return "NPP_MASK_SIZE_15_X_15";
        }
        return "INVALID NppiMaskSize: " + m;
    }

    /**
     * Private constructor to prevent instantiation.
     */
    private NppiMaskSize()
    {
    }
    
}
